@@ -1,6 +1,6 @@
 /*
  *    HardInfo - Displays System Information
- *    Copyright (C) 2003-2007 Leandro A. F. Pereira <leandro@linuxmag.com.br>
+ *    Copyright (C) 2003-2007 Leandro A. F. Pereira <leandro@hardinfo.org>
  *
  *    This program is free software; you can redistribute it and/or modify
  *    it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@ scan_languages(OperatingSystem * os)
     while (fgets(buf, 512, locale)) {
 	if (!strncmp(buf, "locale:", 7)) {
 	    sscanf(buf, "locale: %s", name);
-	    fgets(buf, 128, locale);
+	    (void)fgets(buf, 128, locale);
 	} else if (strchr(buf, '|')) {
 	    gchar **tmp = g_strsplit(buf, "|", 2);
 
@@ -77,7 +77,7 @@ scan_languages(OperatingSystem * os)
 					 "Revision=%s\n"
 					 "Date=%s\n"
 					 "Codeset=%s\n",
-					 FIELD(name), FIELD(title),
+					 name, FIELD(title),
 					 FIELD(source), FIELD(address),
 					 FIELD(email), FIELD(language),
 					 FIELD(territory), FIELD(revision),
