@@ -1,6 +1,6 @@
 /*
  *    HardInfo - Displays System Information
- *    Copyright (C) 2003-2006 Leandro A. F. Pereira <leandro@linuxmag.com.br>
+ *    Copyright (C) 2003-2006 Leandro A. F. Pereira <leandro@hardinfo.org>
  *
  *    This program is free software; you can redistribute it and/or modify
  *    it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 struct _Processor {
     gchar *model_name;
     gchar *flags;
-    gfloat bogomips;
+    gfloat bogomips, cpu_mhz;
 
     gchar *has_fpu;
 };
@@ -53,6 +53,8 @@ __scan_processors(void)
 	g_strfreev(tmp);
     }
 
+    processor->cpu_mhz = 0.0f;
+
     fclose(cpuinfo);
 
     return g_slist_append(NULL, processor);
