@@ -1,6 +1,6 @@
 /*
  *    HardInfo - Displays System Information
- *    Copyright (C) 2003-2006 Leandro A. F. Pereira <leandro@linuxmag.com.br>
+ *    Copyright (C) 2003-2006 Leandro A. F. Pereira <leandro@hardinfo.org>
  *
  *    This program is free software; you can redistribute it and/or modify
  *    it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 struct _Processor {
     gchar *model_name;
     gchar *has_fpu;
+    gfloat cpu_mhz;
 };
 
 static GSList *
@@ -48,6 +49,8 @@ __scan_processors(void)
 
     fclose(cpuinfo);
 
+    processor->cpu_mhz = 0.0f;
+
     return g_slist_append(NULL, processor);
 }
 
