@@ -1,6 +1,6 @@
 /*
  *    HardInfo - Displays System Information
- *    Copyright (C) 2003-2007 Leandro A. F. Pereira <leandro@linuxmag.com.br>
+ *    Copyright (C) 2003-2009 Leandro A. F. Pereira <leandro@hardinfo.org>
  *
  *    This program is free software; you can redistribute it and/or modify
  *    it under the terms of the GNU General Public License as published by
@@ -195,20 +195,25 @@ void cb_about()
 	"",
 	"Contributors:",
 	"Agney Lopes Roth Ferraz",
-	"SCSI support by Pascal F. Martin",
+	"Andrey Esin",
 	"",
 	"Based on work by:",
 	"MD5 implementation by Colin Plumb (see md5.c for details)",
-	"SHA1 implementation by Steve Raid (see sha1.c for details)",
+	"SHA1 implementation by Steve Reid (see sha1.c for details)",
 	"Blowfish implementation by Paul Kocher (see blowfich.c for details)",
 	"Raytracing benchmark by John Walker (see fbench.c for details)",
+	"FFT benchmark by Scott Robert Ladd (see fftbench.c for details)",
 	"Some code partly based on x86cpucaps by Osamu Kayasono",
 	"Vendor list based on GtkSysInfo by Pissens Sebastien",
+	"DMI support based on code by Stewart Adam",
+	"SCSI support based on code by Pascal F. Martin",
 	NULL
     };
     const gchar *artists[] = {
-	"The GNOME Project",
+	"Jakub Szypulka",
 	"Tango Project",
+	"The GNOME Project",
+	"VMWare, Inc. (USB icon from VMWare Workstation 6)",
 	NULL
     };
 
@@ -216,7 +221,7 @@ void cb_about()
     gtk_about_dialog_set_name(GTK_ABOUT_DIALOG(about), "HardInfo");
     gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(about), VERSION);
     gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(about),
-				   "Copyright \302\251 2003-2007 "
+				   "Copyright \302\251 2003-2009 "
 				   "Leandro A. F. Pereira");
     gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(about),
 				  "System information and benchmark tool");
