@@ -1,6 +1,6 @@
 /*
  *    HardInfo - Displays System Information
- *    Copyright (C) 2003-2007 Leandro A. F. Pereira <leandro@linuxmag.com.br>
+ *    Copyright (C) 2003-2007 Leandro A. F. Pereira <leandro@hardinfo.org>
  *
  *    This program is free software; you can redistribute it and/or modify
  *    it under the terms of the GNU General Public License as published by
@@ -26,15 +26,23 @@ static struct {
     { DB_PREFIX "debian_version",	"deb"  },
     { DB_PREFIX "slackware-version",	"slk"  },
     { DB_PREFIX "mandrake-release",	"mdk"  },
+    { DB_PREFIX "mandriva-release",     "mdv"  },
+    { DB_PREFIX "fedora-release",       "fdra" },
+    { DB_PREFIX "coas",                 "coas" },
+    { DB_PREFIX "environment.corel",    "corel"},
     { DB_PREFIX "gentoo-release",	"gnt"  },
     { DB_PREFIX "conectiva-release",	"cnc"  },
     { DB_PREFIX "versão-conectiva",	"cnc"  },
     { DB_PREFIX "turbolinux-release",	"tl"   },
     { DB_PREFIX "yellowdog-release",	"yd"   },
+    { DB_PREFIX "sabayon-release",      "sbn"  },
+    { DB_PREFIX "arch-release",         "arch" },
+    { DB_PREFIX "enlisy-release",       "enlsy"},
     { DB_PREFIX "SuSE-release",		"suse" },
     { DB_PREFIX "sun-release",		"sun"  },
     { DB_PREFIX "zenwalk-version",	"zen"  },
     { DB_PREFIX "puppyversion",		"ppy"  },
+    { DB_PREFIX "distro-release",	"fl"   },
     /*
      * RedHat must be the *last* one to be checked, since
      * some distros (like Mandrake) includes a redhat-relase
@@ -127,6 +135,7 @@ struct _Processor {
 struct _OperatingSystem {
     gchar *kernel;
     gchar *libc;
+    gchar *gcc;
     gchar *distrocode, *distro;
     gchar *hostname;
     gchar *language;
