@@ -1,6 +1,6 @@
 /*
  *    HardInfo - Displays System Information
- *    Copyright (C) 2003-2007 Leandro A. F. Pereira <leandro@linuxmag.com.br>
+ *    Copyright (C) 2003-2007 Leandro A. F. Pereira <leandro@hardinfo.org>
  *
  *    This program is free software; you can redistribute it and/or modify
  *    it under the terms of the GNU General Public License as published by
@@ -48,6 +48,7 @@ typedef enum {
     SHELL_VIEW_DUAL,
     SHELL_VIEW_LOAD_GRAPH,
     SHELL_VIEW_PROGRESS,
+    SHELL_VIEW_PROGRESS_DUAL,
     SHELL_VIEW_N_VIEWS
 } ShellViewType;
 
@@ -65,6 +66,8 @@ typedef enum {
     INFO_TREE_COL_DATA,
     INFO_TREE_COL_PBUF,
     INFO_TREE_COL_PROGRESS,
+    INFO_TREE_COL_EXTRA1,
+    INFO_TREE_COL_EXTRA2,
     INFO_TREE_NCOL
 } ShellInfoTreeColumns;
 
@@ -84,6 +87,7 @@ struct _Shell {
     GtkUIManager	*ui_manager;
 
     ShellViewType	 view_type;
+    gboolean		 normalize_percentage;
     
     gint		_pulses;
     ShellOrderType	_order_type;
@@ -104,7 +108,7 @@ struct _ShellInfoTree {
     GtkTreeModel        *model;
     GtkTreeSelection	*selection;
     
-    GtkTreeViewColumn	 *col_progress, *col_value;
+    GtkTreeViewColumn	 *col_progress, *col_value, *col_extra1, *col_extra2, *col_textvalue;
 };
 
 struct _ShellNote {
@@ -124,7 +128,7 @@ struct _ShellModule {
 
 struct _ShellModuleMethod {
     gchar	*name;
-    gchar	*(*function) (void);
+    gpointer	function;
 };
 
 struct _ShellModuleEntry {
